import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

class DPin extends StatefulWidget {
  const DPin(
      {

      /// required
      required this.number,

      /// by default false
      this.hidden = false,

      /// by default  Color(0xff9c1515)
      this.fieldBorderColor = const Color(0xff9c1515),

      /// by default Verify
      this.buttonText = "Verify",

      /// by default Colors.blue
      this.buttonColor = Colors.blue,

      /// by default Colors.white
      this.buttonTextColor = Colors.white,

      /// by default 100.0,
      this.buttonWidth = 100.0,

      /// by default false
      this.withButton = false,

      /// by default false
      this.underline = false,

      /// required
      required this.onValueChanged});

  /// How many box you need
  final int number;

  /// visible or secured
  final bool hidden;

  /// Box color
  final Color fieldBorderColor;

  /// Button text
  final String buttonText;

  /// Button color
  final Color buttonColor;

  /// Button text color
  final Color buttonTextColor;

  /// Button width
  final double buttonWidth;

  /// Box or underline
  final bool underline;

  /// With button or last button filled
  final bool withButton;

  /// Return result
  final ValueChanged<int> onValueChanged;
  @override
  DPinState createState() => DPinState();
}

class DPinState extends State<DPin> {
  int sum = 0;
  @override
  void initState() {
    setState(() {
      sum = 0;
    });
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    /// list of FocusNode
    final _focusNodes = List.generate(widget.number, (index) => FocusNode());

    /// list of TextEditingController
    final _controller =
        List.generate(widget.number, (index) => TextEditingController());
    return SizedBox(
      height: 150,
      width: MediaQuery.of(context).size.width,
      child: Column(
        mainAxisAlignment: MainAxisAlignment.spaceBetween,
        children: [
          Row(
            mainAxisAlignment: MainAxisAlignment.spaceAround,
            children: [
              for (var i = 0; i < widget.number; i++)
                SizedBox(
                  height: 50,
                  width: 40,
                  child: Center(
                    child: TextField(
                      focusNode: _focusNodes[i],
                      controller: _controller[i],
                      keyboardType: TextInputType.number,
                      inputFormatters: <TextInputFormatter>[
                        FilteringTextInputFormatter.allow(RegExp(r'[0.0-9.9]')),
                      ],
                      textAlign: TextAlign.center,
                      decoration: InputDecoration(
                        counterText: "",
                        enabledBorder: widget.underline
                            ? UnderlineInputBorder(
                                borderSide:
                                    BorderSide(color: widget.fieldBorderColor),
                              )
                            : OutlineInputBorder(
                                borderSide:
                                    BorderSide(color: widget.fieldBorderColor),
                              ),
                        border: widget.underline
                            ? UnderlineInputBorder(
                                borderSide:
                                    BorderSide(color: widget.fieldBorderColor),
                              )
                            : OutlineInputBorder(
                                borderSide:
                                    BorderSide(color: widget.fieldBorderColor),
                              ),
                        errorBorder: widget.underline
                            ? UnderlineInputBorder(
                                borderSide:
                                    BorderSide(color: widget.fieldBorderColor),
                              )
                            : OutlineInputBorder(
                                borderSide:
                                    BorderSide(color: widget.fieldBorderColor),
                              ),
                        focusedBorder: widget.underline
                            ? UnderlineInputBorder(
                                borderSide:
                                    BorderSide(color: widget.fieldBorderColor),
                              )
                            : OutlineInputBorder(
                                borderSide:
                                    BorderSide(color: widget.fieldBorderColor),
                              ),
                      ),
                      maxLength: 1,
                      obscureText: widget.hidden,
                      onChanged: (text) {
                        if (i < widget.number - 1) {
                          if (text.isEmpty) {
                            _focusNodes[i - 1].requestFocus();
                          } else {
                            _focusNodes[i + 1].requestFocus();
                          }
                        } else if (i == widget.number - 1) {
                          for (var ins = 0; ins < widget.number; ins++) {
                            sum = int.parse(
                                sum.toString() + _controller[ins].text);
                          }
                          if (!widget.withButton) {
                            getValue(sum).then((_) => sum = 0);
                          }

                          FocusScope.of(context).requestFocus(FocusNode());
                        }
                      },
                    ),
                  ),
                ),
            ],
          ),
          widget.withButton
              ? SizedBox(
                  width: widget.buttonWidth,
                  child: ElevatedButton(
                    style:
                        ElevatedButton.styleFrom(primary: widget.buttonColor),
                    child: Text(
                      widget.buttonText,
                      style: TextStyle(color: widget.buttonTextColor),
                    ),
                    onPressed: () {
                      for (var ins = 0; ins < widget.number; ins++) {
                        sum = int.parse(sum.toString() + _controller[ins].text);
                      }
                      widget.onValueChanged.call(sum);
                      setState(() {
                        sum = 0;
                      });
                    },
                  ),
                )
              : const SizedBox()
        ],
      ),
    );
  }

  Future<void> getValue(s) async {
    widget.onValueChanged.call(s);
  }
}
